/* eslint-env node */
/* global module: false, process: false */
'use strict';

// 该常量用于监测当前运行环境是否为 'production'，某些配置选项会根据运行环境决定是否开启
const isProduction = process.env.EMBER_ENV === 'production';

module.exports = {
  // 本地模块化样式选项
  cssModules: {
    // PostCSS 选项
    postcssOptions: {
      // 使用 PostCSS 生成样式的 sourcemaps，用于浏览器内对应模块的跳转（推荐）
      map: !isProduction,
    },
  },

  /*
   * NPM Packages 导入管理选项
   *
   * 注意：请不要轻易修改这个对象，如果要禁用某些资源的加载，请修改随后对应的开关
   */
  nodeAssets: {
    // 导入 normalize.css 文件 - 是否编译取决于 useNormalizeCSS 选项的值（推荐）
    'normalize.css': function() {
      return {
        vendor: {include: ['normalize.css']},
        enabled: this.options.useNormalizeCSS,
      };
    },

    // 导入 device.css 文件 - 是否编译取决于 device.js 选项的值
    'device.js': function() {
      return {
        srcDir: 'lib',
        vendor: {include: ['device.js']},
        enabled: this.options.useDeviceJS,
      };
    },
  },

  // 是否默认包含 normalize.css 提供的样式（推荐）
  useNormalizeCSS: true,

  // 是否默认包含 device.js
  useDeviceJS: false,

  // 是否将应用的环境选项作为 <meta> 标签注入到 HTML 文档中，否则编译到最终生成的 js 代码中
  storeConfigInMeta: false,
};
